"""Constants for netbox_onboarding plugin."""

NETMIKO_TO_NAPALM_STATIC = {
    "cisco_ios": "ios",
    "cisco_nxos": "nxos_ssh",
    "arista_eos": "eos",
    "juniper_junos": "junos",
    "cisco_xr": "iosxr",
}
